package com.persagy.gis.json

import com.fasterxml.jackson.core.JsonParser
import com.fasterxml.jackson.core.JsonToken
import com.fasterxml.jackson.databind.DeserializationContext
import com.fasterxml.jackson.databind.JsonDeserializer
import com.persagy.gis.entity.SGisPosition

/**
 *   反序列化
 *
 * @author  张维新 <zhangweixin@sagacloud.cn>
 */
class SGisPositionDeserializer : JsonDeserializer<SGisPosition>() {
    /**
     *
     */
    override fun deserialize(parser: JsonParser, ctxt: DeserializationContext): SGisPosition? {
        // 如果不是数组类型，则返回空
        if (!parser.isExpectedStartArrayToken) {
            return null
        }

        // 存储读取的元素值
        val valueList = ArrayList<Double>()

        var token: JsonToken

        // 依次读取数值型对象
        do  {
            token = parser.nextToken()

            // 如果是数值类型，则保存该值
            if (token.isNumeric) {
                valueList.add(parser.valueAsDouble)
            }
        } while(token != JsonToken.END_ARRAY)

        // 根据读取的数值创建并返回 SGisPosition对象
        return when (valueList.size) {
            0 -> null
            1 -> SGisPosition(valueList[0])
            2 -> SGisPosition(valueList[0], valueList[1])
            else ->
                SGisPosition(valueList[0], valueList[1], valueList[2])
        }
    }
}